/*------------------------------ file.cc -----------------------------
  File classes allow sequential access to external files.
  InFile and OutFile are derived from the base class File.
  These classes provide a simple interface of the more complex
  Fstream class hierarchy supported by most C++ compilers.
Note: to run test driver, #define TEST_FILE at end of file.cc
  Programmer: Glenn D. Blank  
  For chapters 7 and 14 of "The Universal Machine"
  Copyright (C) WCB/McGraw-Hill, 1998
---------------------------------------------------------------------*/
#include <stdlib.h>
#include "file.h"

bool File::exists() const  //has file been successfully opened?
{
  if (Fstream)
    return true;
  else
  return false;
}

void File::close()  //explicitly close a file
{
  Fstream.close();
}

void File::setFileName(const Lstring& name) //set fileName
{
  fileName = name;
}

Lstring& File::getFileName()          //get FileName
{
  return fileName;
}

int File::getLineNumber() const      //get current line #
{
  return lineNumber;
}

InFile::InFile(Lstring name, int maxBuf)
// create a buffer of length maxBuf
{
  if (name.length() > 0)        //non-empty name?
    open(name);                 //open the file with this file name
  bufCapacity = maxBuf;		//size of input buffer
  buf = new char[bufCapacity];	//allocate buf
}

InFile::~InFile()       // close the file
//~InFile() = Fstream.close() AND delete buf
{
  if (exists())
    Fstream.close();
  delete [] buf;
}

void InFile::open(Lstring& name)  // open a file
//open an iFstream with filename name, and reset currentLine
{
  Fstream.open(name.c_str(), ios::in);   //open Fstream for input
  setFileName(name); 	//remember file name
  lineNumber = 0;	//before first readLn(), which increments it
}

Lstring InFile::readLn()       //read a line from input file and return buf
{
  if (!exists() || Fstream.eof())   //no more to read from InFile
    return Lstring("");          //  so, return empty string
  Fstream.getline(buf,bufCapacity); //read a line into buf of Fstream
  lineNumber++; 		    //increment lineNumber
  return Lstring(buf);		    //return buf as a Lstring
}

bool InFile::eof() const      //reached end of file?
{
  return (bool) Fstream.eof();
}

int InFile::bufSize() const //get capacity of buffer
{
  return bufCapacity;
}

OutFile::OutFile(Lstring name)
//Open an output file stream with fileName
{
  if (name.length() > 0) //non-empty name?
    open(name);  //open the file with this file name
}

OutFile::~OutFile() //close the file
{
  Fstream.close();
}

void OutFile::open(Lstring& fileName)  //open a file for writing
{
  Fstream.open(fileName.c_str(),ios::out);
}

void OutFile::nl()                    //put a newline
{
  Fstream << endl;
}

void OutFile::writeLn(const Lstring& str)    //put an Lstring + newline
{
  Fstream << str << endl;
}

OutFile& OutFile::operator << (int x)       //put an int
{
  Fstream << x;
  return *this;
}

OutFile& OutFile::operator << (float x)     //put a float
{
  Fstream << x;
  return *this;
}

OutFile& OutFile::operator << (char x)      //put a char
{
  Fstream << x;
  return *this;
}

OutFile& OutFile::operator << (const Lstring& x)   //put an Lstring
{
  Fstream << x;
  return *this;
}

//#define TEST_FILE
#include <iostream.h>
#ifdef TEST_FILE
void main()	//test valid behaviors of InFile
{ InFile INFILE; //Test constructor with no parameters
  INFILE.open("file.h");	//Test open()
  cout << "Successfully opened " << INFILE.fileName() << endl;
  while ( !INFILE.eof() )
    cout << INFILE.getLineNumber() << ": " << INFILE.readLn() << endl;
  INFILE.close();


  //Opening and closing a non-existant file should not pose problems
  InFile bad;
  bad.open("non-existant");
  cout << "bad.exists()==" << bad.exists() << endl;
  bad.readLn();

  //Test OUTFILE
  OutFile OUTFILE("TEST.OUT");
  OUTFILE.writeLn("Hello, world!");
  int j=2; float x=3.3;
  OUTFILE << j << ',' << x << "i";
  OUTFILE.nl();
  OUTFILE.close();
  //Re-open INFILE in order to read in OUTFILE's output
  INFILE.open("TEST.OUT");
  while ( !INFILE.eof() )
    cout << INFILE.getLineNumber() << ": " << INFILE.readLn() << endl;

}
#endif
