// -*- C++ -*-
#ifndef FILE_H
#define FILE_H
/*------------------------------- FILE.H --------------------------------
  File classes allow sequential access to external files.
  InFile and OutFile are derived from the base class File.
  These classes provide a simple interface of the more complex
  fstream class hierarchy supported by most C++ compilers.
  Note: to run test driver, #define TEST_FILE at end of file.cpp
-----------------------------------------------------------------------*/
#include "..\libfiles\Lstring.h"
#include <fstream.h>

class File { // File is a base class describing operations that the derived classes
// InFile and OutFile have in common
  public:
    bool exists() const; //has file been successfully opened?
    void close();  //explicitly close a file (usually done by destructors)
    void setFileName(const Lstring& name); //set the fileName
    int getLineNumber() const;   //get current line # for File
    Lstring& getFileName();      //get fileName
  protected:
    fstream Fstream; //file object (from fstream class hierarchy)
    Lstring fileName;  //name of file
    int lineNumber;  //current line number
/*Preconditions:
	File.open(name) != NULL
  Axioms: --assume primitives from <fstream.h>
	open(name) => File.open(name) AND fileName = name
	getFileName() = fileName
	lineNumber() = lineNumber
*/
};

class InFile : public File {
// Reading input from an external file
  public:
    //Constructing InFile opens an ifstream, destructing it closes it
    InFile(Lstring name="", int maxBuf=256);
      //open an input file stream with name & buffer of length maxBuf
    ~InFile();  //close the file
    void open(Lstring& name);   //open a file for reading
    Lstring readLn();     // read a line from input file and return buf
    bool eof() const;    //reached end of file?
    int bufSize() const; //get capacity of buffer
  protected:
    char* buf;		 //input buffer
    int bufCapacity;	 //capacity of input buffer

/*Preconditions:
	InFile(char* name) == File.open(name) != NULL
	readLn() == !eof()
  Axioms: --assume primitives from <fstream.h>
	InFile(maxBuf) => bufCapacity = maxBuf
				  AND buf = new char[bufCapacity]
				  AND lineNumber = 0
	open(name) => inFile.open(name) AND fileName = name
	~InFile() = inFile.close() AND delete buf
	readLn() = inFile.getline(buf,bufCapacity) AND lineNumber++
	eof() = fgets(file,bufCapacity,buf) == NULL
*/
};

class OutFile : public File {
//Writing output to an external file
// Comment to make the parser happy
  public:
   //Constructing OutFile opens an ofstream , destructing it closes it
    OutFile(Lstring name="");
      //Open an output file stream with filenName
    ~OutFile(); //close the file
    void open(Lstring& name);  //open a file for writing
    void nl();   		    //put a newline
    void writeLn(const Lstring&);    //put a Lstring + newline
#ifndef LOOKOUT
    OutFile& operator << (int);     //put an int
    OutFile& operator << (float);   //put a float
    OutFile& operator << (char);    //put a char
    OutFile& operator << (const Lstring&);   //put a Lstring
#endif
};
#endif

