/*------------------------------ note.cc -----------------------------
  The implementation of the Note class, which gets a PC's internal
    speaker (not the sound card) to play musical notes.
    Lots of obscure implementation details have been hidden away
    in this file!
  Programmer: Jesse Thilo
  For chapter 5 and following of "The Universal Machine"
  Copyright (C) WCB/McGraw-Hill, 1998
---------------------------------------------------------------------*/
#include <stdlib.h>
#ifdef __BORLANDC__
#else
#  include <pc.h>
#endif
#include <dos.h>
#include <iostream.h>
#include <ctype.h>
#include "note.h"

#define PANIC exit(1) //abort on error
//#define PANIC //ignore errors

#define ONE_MINUTE (60000L) // milliseconds in one minute

typedef int note_t;
typedef int sustain_t;
typedef int tempo_t;
		    
enum {
  legato = -1, normal, mezzo_staccato, staccato, staccatissimo
};

enum {
  largo = 40, largetto = 70, adagio = 98, andante = 126, allegro = 150,
  presto = 180, prestissimo = 210
};

static sustain_t sustain = normal;
static tempo_t tempo = andante;

static struct {
  const char *note_name;
  unsigned note_freq;
} note_assoc[] = {
  { "C",    8372 }, { "C+",   8870 },
  { "D-",   8870 }, { "D",    9397 }, { "D+",   9956 },
  { "E-",   9956 }, { "E",   10548 },
  { "F",   11175 }, { "F+",  11840 },
  { "G-",  11840 }, { "G",   12544 }, { "G+",  13290 },
  { "A-",  13290 }, { "A",   14080 }, { "A+",  14917 },
  { "B-",  14917 }, { "B",   15804 },
  { " ", 0 }, { NULL, 0 },
};

Note::Note(Lstring nam, int oct)
{
  int i = 0;

  note = -1;
  while (note == -1 && note_assoc[i].note_name != NULL) {
    if (strcmp(note_assoc[i].note_name, nam.c_str()) == 0)
      note = i;
    i++;
  }

  if (note == -1) {
    cerr << "Note created with invalid name:\n"
         << "  Note(\"" << nam.c_str() << "\", " << oct << ")"
         << endl;
    note = i - 1;     // default to a rest.
    PANIC;
  }

  name = new char[strlen(note_assoc[note].note_name) + 1];
  strcpy(name, note_assoc[note].note_name);
  duration = 1.0;
  setOctave(oct);
}

void quiet(void)
{
  nosound(); // this might be a macro
}

void Note::play(void)
{
  static bool music_mode = false;
  unsigned total_time, play_time, quiet_time;
  
  if (music_mode == false) {
    atexit(quiet);
    music_mode = true;
  }

  total_time = (unsigned)((ONE_MINUTE / tempo) * duration);
  play_time = (sustain == legato) ? (4 * total_time) :
              ((4 - sustain) * total_time - 2);
  quiet_time = 4 * total_time - play_time;
  
  if (frequency)
    sound(frequency);
  else
    nosound();
  delay(play_time);
  if (sustain != legato) {
    nosound();
    delay(quiet_time);
  }
}

void Note::show(void)
{
  cout << name << octave << ' ' << flush;
}

void Note::setOctave(int oct)
{
  if (oct < 0 || oct > 9) {
    cerr << "Note octave out of range:\n"
         << "  setOctave(" << oct << ")"
         << endl;
    PANIC;
  }
  else {
    octave = oct;
    compute_frequency();
  }
}

int Note::getOctave()
{
  return octave;
}

void Note::whole()
{
  duration = 1.0;
}

void Note::half()
{
  duration = 0.5;
}

void Note::quarter()
{
  duration = 0.25;
}

void Note::eighth()
{
  duration = 0.125;
}

void Note::dot()
{
  duration *= 1.5;
}

void Note::triplet()
{
  duration *= 2.0 / 3.0;
}

void Note::compute_frequency(void)
{
  frequency = note_assoc[note].note_freq;

  if (frequency != 0)
    frequency >>= 9 - octave;
}



